@@ -5,16 +5,16 @@ use Test::More;
 
 our @EXPORT = qw(
     ok is isnt like unlike is_deeply cmp_ok
-    skip todo todo_skip pass fail
+    skip todo_skip pass fail
     eq_array eq_hash eq_set
     plan can_ok isa_ok diag
     $TODO
 
     chunks delimiters spec_file spec_string filters filters_map 
-    run run_is
-    diff_is
+    run run_is run_like
     WWW XXX YYY ZZZ
 );
+#     diff_is
 
 # XXX Add these manually for now
 sub WWW() { goto &Spiffy::WWW }
@@ -22,7 +22,7 @@ sub XXX() { goto &Spiffy::XXX }
 sub YYY() { goto &Spiffy::YYY }
 sub ZZZ() { goto &Spiffy::ZZZ }
 
-our $VERSION = '0.16';
+our $VERSION = '0.17';
 
 sub import() {
     _strict_warnings();
@@ -122,7 +122,9 @@ sub filters_map() {
 }
 
 sub run(&) {
-    my $self = $default_object;
+    my $self = ref($_[0]) eq __PACKAGE__
+    ? shift
+    : $default_object;
     my $callback = shift;
     for my $chunk ($self->chunks) {
         &{$callback}($chunk);
@@ -130,7 +132,9 @@ sub run(&) {
 }
 
 sub run_is($$) {
-    my $self = $default_object;
+    my $self = ref($_[0]) eq __PACKAGE__
+    ? shift
+    : $default_object;
     my ($x, $y) = @_;
     for my $chunk ($self->chunks) {
         is($chunk->$x, $chunk->$y, 
@@ -139,12 +143,25 @@ sub run_is($$) {
     }
 }
 
-# XXX Dummy implementation for now.
-sub diff_is($$;$) {
-    require Algorithm::Diff;
-    is($_[0], $_[1], (@_ > 1 ? ($_[2]) : ()));
+sub run_like($$) {
+    my $self = ref($_[0]) eq __PACKAGE__
+    ? shift
+    : $default_object;
+    my ($x, $y) = @_;
+    for my $chunk ($self->chunks) {
+        my $regexp = ref $y ? $y : $chunk->$y;
+        like($chunk->$x, $regexp,
+             $chunk->description ? $chunk->description : ()
+            );
+    }
 }
 
+# XXX Dummy implementation for now.
+# sub diff_is($$;$) {
+#     require Algorithm::Diff;
+#     is($_[0], $_[1], (@_ > 1 ? ($_[2]) : ()));
+# }
+
 sub _chunks_init {
     my $spec = $self->spec;
     my $cd = $self->chunk_delim;
@@ -172,7 +189,21 @@ sub _make_chunk {
     while (@parts) {
         my ($type, $filters, $text) = splice(@parts, 0, 3);
         for my $filter ($self->_get_filters($type, $filters)) {
-            $text = $self->filter_class->$filter($text);
+            my @args = ();
+            if ($filter =~ s/=(.*)$//) {
+                @args = ($1);
+            }
+            my $function = "main::$filter";
+            no strict 'refs';
+            if (defined &$function) {
+                $text = &$function($text, @args);
+            }
+            elsif ($self->filter_class->can($filter)) {
+                $text = $self->filter_class->$filter($text, @args);
+            }
+            else {
+                die "Can't find a function or method for '$filter' filter\n";
+            }
         }
         $chunk->set_chunk($type, $text);
     }
@@ -320,6 +351,7 @@ sub list {
 }
 
 sub dumper {
+    no warnings 'once';
     require Data::Dumper;
     local $Data::Dumper::Sortkeys = 1;
     local $Data::Dumper::Indent = 1;
@@ -334,6 +366,27 @@ use warnings;
 ...
 }
 
+sub regexp {
+    my ($text, $flags) = (@_, '');
+    if ($text =~ /\n.*?\n/s) {
+        $flags .= 'x'
+          unless $flags =~ /x/;
+    }
+    else {
+        CORE::chomp($text);
+    }
+    my $regexp = eval "qr{$text}$flags";
+    die $@ if $@;
+    return $regexp;
+}
+
+sub get_url {
+    my $url = shift;
+    CORE::chomp($url);
+    require LWP::Simple;
+    LWP::Simple::get($url);
+}
+    
 __DATA__
 
 =head1 NAME
@@ -351,7 +404,7 @@ Test::Chunks - Chunky Data Driven Testing Support
     for my $chunk (chunks) {
         # Note that this code is conceptual only. Pod::Simple is not so
         # simple as to provide a simple pod_to_html function.
-        diff_is(
+        is(
             Pod::Simple::pod_to_html($chunk->pod),
             $chunk->text,
             $chunk->description, 
@@ -395,17 +448,6 @@ Test::Chunks extends Test::More and exports all of its functions. So you
 can basically write your tests the same as Test::More. Test::Chunks
 exports a few more functions though:
 
-=head2 run(&subroutine)
-
-The C<run> function takes a subroutine as an argument, and calls the sub one
-time for each chunk in the specification. It passes the current chunk object
-to the sub routine.
-
-    run {
-        my $chunk = shift;
-        is(process($chunk->foo), $chunk->bar, $chunk->description);
-    };
-
 =head2 chunks()
 
 The most important function is C<chunks>. In list context it returns a
@@ -418,6 +460,42 @@ Each Test::Chunk object has methods that correspond to the names of that
 object's data sections. There is also a C<description> method for
 accessing the description text of the object.
 
+=head2 run(&subroutine)
+
+There are many ways to write your tests. You can reference each chunk
+individually or you can loop over all the chunks and perform a common
+operation. The C<run> function does the looping for you, so all you need
+to do is pass it a code block to execute for each chunk.
+
+The C<run> function takes a subroutine as an argument, and calls the sub
+one time for each chunk in the specification. It passes the current
+chunk object to the subroutine.
+
+    run {
+        my $chunk = shift;
+        is(process($chunk->foo), $chunk->bar, $chunk->description);
+    };
+
+=head2 run_is(data_name1, data_name2)
+
+Many times you simply want to see if two data sections are equivalent in
+every chunk, probably after having been run through one or more filters.
+With the C<run_is> function, you can just pass the names of any two data
+sections that exist in every chunk, and it will loop over every chunk
+comparing the two sections.
+
+    run_is 'foo', 'bar';
+
+=head2 run_like(data_name, regexp | data_name);
+
+The C<run_like> function is similar to C<run_is> except the second
+argument is a regular expression. The regexp can either be a C<qr{}>
+object or a data section that has been filtered into a regular
+expression.
+
+    run_like 'foo', qr{<html.*};
+    run_like 'foo', 'match';
+
 =head2 delimiters($chunk_delimiter, $data_delimiter)
 
 Override the default delimiters of C<===> and C<--->.
@@ -451,11 +529,13 @@ to an array ref of filters for that data type.
 
 If a filters list has only one element, the array ref is optional.
 
-=head2 diff_is()
+=cut
 
-Like Test::More's C<is()>, but on failure reports a diff of the expected
-and actual output. This is often very useful when your chunks are large.
-Requires the Algorithm::Diff module.
+# =head2 diff_is()
+# 
+# Like Test::More's C<is()>, but on failure reports a diff of the expected
+# and actual output. This is often very useful when your chunks are large.
+# Requires the Algorithm::Diff module.
 
 =head2 default_object()
 
@@ -555,8 +635,10 @@ Example:
 
     use Test::Chunks;
 
-    filters(foo bar);
-    filters_map({ perl => 'strict'});
+    filters qw(foo bar);
+    filters_map { perl => 'strict'};
+
+    sub upper { uc(shift) }
 
     __END__
 
@@ -605,6 +687,18 @@ Same as the C<lines> filter, except all newlines are chomped.
 Run Perl's C<eval> command against the data and use the returned value
 as the data.
 
+=head2 regexp[=xism]
+
+The C<regexp> filter will turn your data section into a regular
+expression object. You can pass in extra flags after an equals sign.
+
+If the text contains more than one line then the 'x' flag is assumed.
+
+=head2 get_url
+
+The text is chomped and considered to be a url. Then LWP::Simple::get is
+used to fetch the contents of the url.
+
 =head2 yaml
 
 Apply the YAML::Load function to the data chunk and use the resultant
@@ -634,20 +728,33 @@ Unescape all backslash escaped chars.
 
 =head2 Rolling Your Own Filters
 
-Creating filter extensions is very simple. Here is a self
-explanatory example:
+Creating filter extensions is very simple. You can either write a
+I<function> in the C<main> namespace, or a I<method> in the
+C<Test::Chunks::Filter> namespace. In either case the text and any
+extra arguments are passed in and you return whatever you want the new
+value to be.
+
+Here is a self explanatory example:
 
     use Test::Chunks;
 
-    filters(foo);
+    filters 'foo', 'bar=xyz';
 
-    sub Test::Chunks::Filter::foo {
+    sub foo {
+        transform(shift);
+    }
+        
+    sub Test::Chunks::Filter::bar {
         my $class = shift;
         my $data = shift;
-        # transform $data in a fooish manner
+        my $args = shift;
+        # transform $data in a barish manner
         return $data;
     }    
 
+Normally you'll probably just use the functional interface, although all
+the builtin filters are methods.
+
 =head1 OO
 
 Test::Chunks has a nice functional interface for simple usage. Under the
@@ -678,10 +785,6 @@ Test::Chunks automatically adds
 
 to all of your test scripts. A Spiffy feature indeed.
 
-=head1 TODO
-
-* diff_is() just calls is() for now. Need to implement.
-
 =head1 AUTHOR
 
 Brian Ingerson <ingy@cpan.org>
